/* eslint-disable guard-for-in */
/* eslint-disable no-undef */
import { LightningElement, api } from "lwc";
import { ShowToastEvent } from "lightning/platformShowToastEvent";
import { loadStyle, loadScript } from "lightning/platformResourceLoader";
import { createRecord, updateRecord, deleteRecord } from "lightning/uiRecordApi";

// Static resources
import SchedulerFiles from "@salesforce/resourceUrl/trial712";

// Controllers
import getEvents from "@salesforce/apex/SchedulerData.getEvents";

function unwrap(fromSF) {
    const data = fromSF.events.map((a) => ({
        id: a.Id,
        info: a.Name,
        start_date: a.Start_Date__c,
        end_date: a.End_Date__c,
        text: a.Text__c,
    }));

    return { data };
}

export default class SchedulerView extends LightningElement {
    static delegatesFocus = true;

    @api height;
    schedulerInitialized = false;

    renderedCallback() {
        if (this.schedulerInitialized) {
            return;
        }
        this.schedulerInitialized = true;

        Promise.all([
            loadScript(this, SchedulerFiles + "/dhtmlxscheduler.js"),
            loadStyle(this, SchedulerFiles + "/dhtmlxscheduler.css")
        ])
            .then(() => {
                this.initializeUI();
            })
            .catch((error) => {
                this.dispatchEvent(
                    new ShowToastEvent({
                        title: "Error loading scheduler",
                        message: error.message,
                        variant: "error"
                    })
                );
            });
    }

    initializeUI() {
        const root = this.template.querySelector(".thescheduler");
        root.style.height = this.height + "px";

        //uncomment the following line if you use the Enterprise or Ultimate version
        const scheduler = window.Scheduler.getSchedulerInstance();
        scheduler.templates.parse_date = (date) => new Date(date);
        scheduler.templates.format_date = (date) => date.toISOString();
        scheduler.plugins({
            timeline: true,
            treetimeline: true
        });
        scheduler.config.header = [
            "day",
            "week",
            "month",
            "timeline",
            "date",
            "prev",
            "today",
            "next"
        ];
        const elements = [ 
            {key:10, label:"Web Testing Dep.", open: true, children: [
                {key:20, label:"Elizabeth Taylor"},
                {key:30, label:"Managers",  children: [
                    {key:40, label:"John Williams"},
                    {key:50, label:"David Miller"}
                ]},
                {key:60, label:"Linda Brown"},
                {key:70, label:"George Lucas"}
            ]},
            {key:1101, label:"Human Relations Dep.", open:true, children: [
                {key:80, label:"Kate Moss"},
                {key:81, label:"Dian Fossey"},
                {key:82, label:"Kate Moss"},
                {key:83, label:"Dian Fossey"},
                {key:84, label:"Kate Moss"},
                {key:85, label:"Dian Fossey"},
                {key:86, label:"Kate Moss"},
                {key:87, label:"Dian Fossey"},
                {key:88, label:"Kate Moss"},
                {key:89, label:"Dian Fossey"},
                {key:90, label:"Kate Moss"},
                {key:91, label:"Dian Fossey"},
                {key:92, label:"Kate Moss"},
                {key:93, label:"Dian Fossey"},
                {key:94, label:"Kate Moss"},
                {key:95, label:"Dian Fossey"},
                {key:96, label:"Kate Moss"},
                {key:97, label:"Dian Fossey"},
                {key:98, label:"Kate Moss"},
                {key:99, label:"Dian Fossey"},
                {key:100, label:"Kate Moss"},
                {key:101, label:"Dian Fossey"},
                {key:102, label:"Kate Moss"},
                {key:103, label:"Dian Fossey"},
                {key:104, label:"Kate Moss"},
                {key:105, label:"Dian Fossey"},
                {key:106, label:"Kate Moss"},
                {key:107, label:"Dian Fossey"},
                {key:108, label:"Kate Moss"},
                {key:109, label:"Dian Fossey"},
                {key:110, label:"Kate Moss"},
                {key:111, label:"Dian Fossey"},
                {key:112, label:"Kate Moss"},
                {key:113, label:"Dian Fossey"},
                {key:114, label:"Kate Moss"},
                {key:115, label:"Dian Fossey"},
                {key:116, label:"Kate Moss"},
                {key:117, label:"Dian Fossey"}
            ]}
        ];
        
        scheduler.createTimelineView({
            section_autoheight: false,
            name:	"timeline",
            x_unit:	"minute",
            x_date:	"%H:%i",
            x_step:	30,
            x_size: 24,
            x_start: 16,
            x_length:	48,
            y_unit: elements,
            y_property:	"section_id",
            render: "tree",
            folder_dy:30,
            dy:60,
            smart_rendering: true
        });
        scheduler.init(root, new Date(), "timeline");

        getEvents().then((d) => {

            const chartData = unwrap(d);
            scheduler.parse({
                events: chartData.data,
            });
        });

        ///↓↓↓ saving changes back to SF backend ↓↓↓
        scheduler.createDataProcessor(function (entity, action, data, id) {
            switch (action) {
                case "create":
                    console.log("createEvent", data);
                    const insert = {
                        apiName: "SchedulerEvent__c",
                        fields: {
                            Name: data.info,
                            Start_Date__c: data.start_date,
                            End_Date__c: data.end_date,
                            Text__c: data.text
                        }
                    };
                    scheduler.config.readonly = true; // suppress changes until saving is complete
                    return createRecord(insert).then((res) => {
                        scheduler.config.readonly = false;
                        return { tid: res.id, ...res };
                    });
                case "update":
                    console.log("updateEvent", data);
                    const update = {
                        fields: {
                            Id: id,
                            Name: data.info,
                            Start_Date__c: data.start_date,
                            End_Date__c: data.end_date,
                            Text__c: data.text
                        }
                    };
                    return updateRecord(update).then(() => ({}));
                case "delete":
                    return deleteRecord(id).then(() => ({}));
            }
        });
    }
}